from enum import Enum

class FILING_TYPES(Enum):

    _1012B='1012B'
    _1012BAD='1012BAD'
    _1012G='1012G'
    _1012GAD='1012GAD'
    _10C='10C'
    _10CAD='10CAD'
    _10D='10D'
    _10DAD='10DAD'
    _10K='10K'
    _10KAD='10KAD'
    _10K405='10K405'
    _10K405AD='10K405AD'
    _10KSB='10KSB'
    _10KSBAD='10KSBAD'
    _10KSB40='10KSB40'
    _10KSB40AD='10KSB40AD'
    _10KT='10KT'
    _10KTAD='10KTAD'
    _10KT405='10KT405'
    _10KT405AD='10KT405AD'
    _10Q='10Q'
    _10QAD='10QAD'
    _10QSB='10QSB'
    _10QSBAD='10QSBAD'
    _10QT='10QT'
    _10QTAD='10QTAD'
    _10SB12B='10SB12B'
    _10SB12BAD='10SB12BAD'
    _10SB12G='10SB12G'
    _10SB12GAD='10SB12GAD'
    _11K='11K'
    _11KAD='11KAD'
    _11KT='11KT'
    _11KTAD='11KTAD'
    _13FE='13FE'
    _13FEAD='13FEAD'
    _13FHR='13FHR'
    _13FHRAD='13FHRAD'
    _13FNT='13FNT'
    _13FNTAD='13FNTAD'
    _144='144'
    _144AD='144AD'
    _1512B='1512B'
    _1512BAD='1512BAD'
    _1512G='1512G'
    _1512GAD='1512GAD'
    _1515D='1515D'
    _1515DAD='1515DAD'
    _15F12B='15F12B'
    _15F12BAD='15F12BAD'
    _15F12G='15F12G'
    _15F12GAD='15F12GAD'
    _15F15D='15F15D'
    _15F15DAD='15F15DAD'
    _1812B='1812B'
    _1812BAD='1812BAD'
    _1812G='1812G'
    _1812GAD='1812GAD'
    _18K='18K'
    _18KAD='18KAD'
    _1A='1A'
    _1AAD='1AAD'
    _1APOS='1APOS'
    _1AW='1AW'
    _1AWAD='1AWAD'
    _1E='1E'
    _1EAD='1EAD'
    _1K='1K'
    _1KAD='1KAD'
    _1SA='1SA'
    _1SAAD='1SAAD'
    _1U='1U'
    _1UAD='1UAD'
    _1Z='1Z'
    _1ZAD='1ZAD'
    _1ZW='1ZW'
    _1ZWAD='1ZWAD'
    _20F='20F'
    _20FAD='20FAD'
    _20FR12B='20FR12B'
    _20FR12BAD='20FR12BAD'
    _20FR12G='20FR12G'
    _20FR12GAD='20FR12GAD'
    _24F2EL='24F2EL'
    _24F2ELAD='24F2ELAD'
    _24F2NT='24F2NT'
    _24F2NTAD='24F2NTAD'
    _24F2TM='24F2TM'
    _24F2TMAD='24F2TMAD'
    _24F3NT='24F3NT'
    _24F3NTAD='24F3NTAD'
    _25='25'
    _25AD='25AD'
    _253G1='253G1'
    _253G2='253G2'
    _253G3='253G3'
    _253G4='253G4'
    _25NSE='25NSE'
    _25NSEAD='25NSEAD'
    _2E='2E'
    _2EAD='2EAD'
    _3='3'
    _3AD='3AD'
    _305B2='305B2'
    _305B2AD='305B2AD'
    _35APP='35APP'
    _35APPAD='35APPAD'
    _35CERT='35CERT'
    _35CERTAD='35CERTAD'
    _4='4'
    _4AD='4AD'
    _4017F1='4017F1'
    _4017F1AD='4017F1AD'
    _4017F2='4017F2'
    _4017F2AD='4017F2AD'
    _4017G='4017G'
    _4017GAD='4017GAD'
    _4017GCS='4017GCS'
    _4017GCSAD='4017GCSAD'
    _4024B2='4024B2'
    _4024B2AD='4024B2AD'
    _4033='4033'
    _4033AD='4033AD'
    _406B='406B'
    _406BAD='406BAD'
    _408B25='408B25'
    _408F2='408F2'
    _408F2AD='408F2AD'
    _408FA='408FA'
    _408FAAD='408FAAD'
    _408FB='408FB'
    _408FBAD='408FBAD'
    _408FL='408FL'
    _408FLAD='408FLAD'
    _408FLA='408FLA'
    _408FLM='408FLM'
    _408FLMA='408FLMA'
    _408FM='408FM'
    _408FMAD='408FMAD'
    _40APP='40APP'
    _40APPAD='40APPAD'
    _40F='40F'
    _40FAD='40FAD'
    _40FR12B='40FR12B'
    _40FR12BAD='40FR12BAD'
    _40FR12G='40FR12G'
    _40FR12GAD='40FR12GAD'
    _40OIP='40OIP'
    _40OIPAD='40OIPAD'
    _424A='424A'
    _424B1='424B1'
    _424B2='424B2'
    _424B3='424B3'
    _424B4='424B4'
    _424B5='424B5'
    _424B7='424B7'
    _424B8='424B8'
    _424H='424H'
    _424HAD='424HAD'
    _425='425'
    _45B3='45B3'
    _45B3AD='45B3AD'
    _485A24E='485A24E'
    _485A24F='485A24F'
    _485APOS='485APOS'
    _485B24E='485B24E'
    _485B24F='485B24F'
    _485BPOS='485BPOS'
    _485BXT='485BXT'
    _485BXTE='485BXTE'
    _485BXTF='485BXTF'
    _486A24F='486A24F'
    _486APOS='486APOS'
    _486B24F='486B24F'
    _486BPOS='486BPOS'
    _487='487'
    _497='497'
    _497AD='497AD'
    _497H2='497H2'
    _497J='497J'
    _497K='497K'
    _497K1='497K1'
    _497K2='497K2'
    _497K3A='497K3A'
    _497K3B='497K3B'
    _5='5'
    _5AD='5AD'
    _6BNTC='6BNTC'
    _6BORDR='6BORDR'
    _6K='6K'
    _6KAD='6KAD'
    _8A12B='8A12B'
    _8A12BAD='8A12BAD'
    _8A12G='8A12G'
    _8A12GAD='8A12GAD'
    _8B12B='8B12B'
    _8B12BAD='8B12BAD'
    _8B12G='8B12G'
    _8F2NTC='8F2NTC'
    _8F2ORDR='8F2ORDR'
    _8K='8K'
    _8KAD='8KAD'
    _8K12B='8K12B'
    _8K12BAD='8K12BAD'
    _8K12G3='8K12G3'
    _8K12G3AD='8K12G3AD'
    _8K15D5='8K15D5'
    _8K15D5AD ='8K15D5AD'
    ABS15G='ABS15G'
    ABS15GAD='ABS15GAD'
    ABSEE='ABSEE'
    ABSEEAD='ABSEEAD'
    ANNLRPT='ANNLRPT'
    ANNLRPTAD='ANNLRPTAD'
    APPNTC='APPNTC'
    APPORDR='APPORDR'
    APPWD='APPWD'
    APPWDAD='APPWDAD'
    ARS='ARS'
    ARSAD='ARSAD'
    AW='AW'
    AWWD='AWWD'
    BW2='BW2'
    BW3='BW3'
    C='C'
    CAD='CAD'
    CAWD='CAWD'
    CAR='CAR'
    CARAD='CARAD'
    CARAWD='CARAWD'
    CARW='CARW'
    CB='CB'
    CBAD='CBAD'
    CFPORTAL='CFPORTAL'
    CFPORTALAD='CFPORTALAD'
    CFPORTALWD='CFPORTALWD'
    CORRESP='CORRESP'
    CTORDER='CTORDER'
    CTR='CTR'
    CTRAD='CTRAD'
    CTRW='CTRW'
    CU='CU'
    CUAD='CUAD'
    CUW='CUW'
    CW='CW'
    CWAD='CWAD'
    D='D'
    DAD='DAD'
    DEF13E3='DEF13E3'
    DEF13E3AD='DEF13E3AD'
    DEF14A='DEF14A'
    DEF14C='DEF14C'
    DEFA14A='DEFA14A'
    DEFA14C='DEFA14C'
    DEFC14A='DEFC14A'
    DEFC14C='DEFC14C'
    DEFM14A='DEFM14A'
    DEFM14C='DEFM14C'
    DEFN14A='DEFN14A'
    DEFR14A='DEFR14A'
    DEFR14C='DEFR14C'
    DEFS14A='DEFS14A'
    DEFS14C='DEFS14C'
    DELAM='DELAM'
    DFAN14A='DFAN14A'
    DFRN14A='DFRN14A'
    DOS='DOS'
    DOSA='DOSA'
    DOSLTR='DOSLTR'
    DRS='DRS'
    DRSAD='DRSAD'
    DRSLTR='DRSLTR'
    DSTRBRPT='DSTRBRPT'
    DSTRBRPTAD='DSTRBRPTAD'
    EFFECT='EFFECT'
    ETR='ETR'
    F1='F1'
    F1AD='F1AD'
    F10='F10'
    F10AD='F10AD'
    F10EF='F10EF'
    F10MEF='F10MEF'
    F10POS='F10POS'
    F1MEF='F1MEF'
    F2='F2'
    F2AD='F2AD'
    F2D='F2D'
    F2DPOS='F2DPOS'
    F2MEF='F2MEF'
    F3='F3'
    F3AD='F3AD'
    F3ASR='F3ASR'
    F3D='F3D'
    F3DPOS='F3DPOS'
    F3MEF='F3MEF'
    F4='F4'
    F4AD='F4AD'
    F4EF='F4EF'
    F4EFAD='F4EFAD'
    F4MEF='F4MEF'
    F4POS='F4POS'
    F6='F6'
    F6AD='F6AD'
    F6EF='F6EF'
    F6POS='F6POS'
    F7='F7'
    F7AD='F7AD'
    F7POS='F7POS'
    F8='F8'
    F8AD='F8AD'
    F80='F80'
    F80AD='F80AD'
    F80POS='F80POS'
    F8POS='F8POS'
    F9='F9'
    F9AD='F9AD'
    F9EF='F9EF'
    F9MEF='F9MEF'
    F9POS='F9POS'
    FN='FN'
    FNAD='FNAD'
    FWP='FWP'
    FX='FX'
    FXAD='FXAD'
    IRANNOTICE='IRANNOTICE'
    MA='MA'
    MAAD='MAAD'
    MAA='MAA'
    MAI='MAI'
    MAIAD='MAIAD'
    MANR='MANR'
    MAW='MAW'
    N1='N1'
    N1AD='N1AD'
    N14='N14'
    N14AD='N14AD'
    N148C='N148C'
    N148CAD='N148CAD'
    N14AE='N14AE'
    N14AEAD='N14AEAD'
    N14AE24='N14AE24'
    N14AE24AD='N14AE24AD'
    N14EL24='N14EL24'
    N14EL24AD='N14EL24AD'
    N14MEF='N14MEF'
    N18F1='N18F1'
    N18F1AD='N18F1AD'
    N1A='N1A'
    N1AAD='N1AAD'
    N1AEL='N1AEL'
    N1AELAD='N1AELAD'
    N2='N2'
    N2AD='N2AD'
    N23C1='N23C1'
    N23C1AD='N23C1AD'
    N23C2='N23C2'
    N23C2AD='N23C2AD'
    N23C3A='N23C3A'
    N23C3AAD='N23C3AAD'
    N23C3B='N23C3B'
    N23C3BAD='N23C3BAD'
    N23C3C='N23C3C'
    N23C3CAD='N23C3CAD'
    N27D1='N27D1'
    N27D1AD='N27D1AD'
    N2MEF='N2MEF'
    N3='N3'
    N3AD='N3AD'
    N30B2='N30B2'
    N30D='N30D'
    N30DAD='N30DAD'
    N3EL='N3EL'
    N3ELAD='N3ELAD'
    N4='N4'
    N4AD='N4AD'
    N4EL='N4EL'
    N4ELAD='N4ELAD'
    N5='N5'
    N5AD='N5AD'
    N54A='N54A'
    N54AAD='N54AAD'
    N54C='N54C'
    N54CAD='N54CAD'
    N6='N6'
    N6AD='N6AD'
    N6C9='N6C9'
    N6C9AD='N6C9AD'
    N6F='N6F'
    N6FAD='N6FAD'
    N8A='N8A'
    N8AAD='N8AAD'
    N8B2='N8B2'
    N8B2AD='N8B2AD'
    N8B3='N8B3'
    N8B3AD='N8B3AD'
    N8B4='N8B4'
    N8B4AD='N8B4AD'
    N8F='N8F'
    N8FAD='N8FAD'
    N8FNTC='N8FNTC'
    N8FORDR='N8FORDR'
    NCEN='NCEN'
    NCENDA='NCENDA'
    NCR='NCR'
    NCRAD='NCRAD'
    NCSR='NCSR'
    NCSRAD='NCSRAD'
    NCSRS='NCSRS'
    NCSRSAD='NCSRSAD'
    NMFP='NMFP'
    NMFPAD='NMFPAD'
    NMFP1='NMFP1'
    NMFP1AD='NMFP1AD'
    NMFP2='NMFP2'
    NMFP2AD='NMFP2AD'
    NPX='NPX'
    NPXAD='NPXAD'
    NQ='NQ'
    NQAD='NQAD'
    NRSROCE='NRSROCE'
    NRSROCEAD='NRSROCEAD'
    NRSROUPD='NRSROUPD'
    NRSROUPDAD='NRSROUPDAD'
    NSARA='NSARA'
    NSARAAD='NSARAAD'
    NSARAT='NSARAT'
    NSARATAD='NSARATAD'
    NSARB='NSARB'
    NSARBAD='NSARBAD'
    NSARBT='NSARBT'
    NSARBTAD='NSARBTAD'
    NSARU='NSARU'
    NSARUAD='NSARUAD'
    NT10D='NT10D'
    NT10DAD='NT10DAD'
    NT10K='NT10K'
    NT10KAD='NT10KAD'
    NT10Q='NT10Q'
    NT10QAD='NT10QAD'
    NT11K='NT11K'
    NT11KAD='NT11KAD'
    NT15D2='NT15D2'
    NT15D2AD='NT15D2AD'
    NT20F='NT20F'
    NT20FAD='NT20FAD'
    NTFNCEN='NTFNCEN'
    NTFNCSR='NTFNCSR'
    NTFNSAR='NTFNSAR'
    NTN10D='NTN10D'
    NTN10K='NTN10K'
    NTN10Q='NTN10Q'
    NTN15D2='NTN15D2'
    NTN20F='NTN20F'
    NTNCEN='NTNCEN'
    NTNCENX='NTNCENX'
    NTNCENXDA='NTNCENXDA'
    NTNCSR='NTNCSR'
    NTNCSRAD='NTNCSRAD'
    NTNMFP='NTNMFP'
    NTNMFPAD='NTNMFPAD'
    NTNMFP1='NTNMFP1'
    NTNMFP1AD='NTNMFP1AD'
    NTNMFP2='NTNMFP2'
    NTNMFP2AD='NTNMFP2AD'
    NTNSAR='NTNSAR'
    NTNSARAD='NTNSARAD'
    OIPNTC='OIPNTC'
    OIPORDR='OIPORDR'
    POS462B='POS462B'
    POS462C='POS462C'
    POS8C='POS8C'
    POSAM='POSAM'
    POSAMC='POSAMC'
    POSAMI='POSAMI'
    POSASR='POSASR'
    POSEX='POSEX'
    PRE14A='PRE14A'
    PRE14C='PRE14C'
    PREC14A='PREC14A'
    PREC14C='PREC14C'
    PREM14A='PREM14A'
    PREM14C='PREM14C'
    PREN14A='PREN14A'
    PRER14A='PRER14A'
    PRER14C='PRER14C'
    PRES14A='PRES14A'
    PRES14C='PRES14C'
    PRRN14A='PRRN14A'
    PX14A6G='PX14A6G'
    PX14A6N='PX14A6N'
    QRTLYRPT='QRTLYRPT'
    QRTLYRPTAD='QRTLYRPTAD'
    QUALIF='QUALIF'
    RW='RW'
    RWWD='RWWD'
    S1='S1'
    S1AD='S1AD'
    S11='S11'
    S11AD='S11AD'
    S11MEF='S11MEF'
    S1MEF='S1MEF'
    S2='S2'
    S2AD='S2AD'
    S20='S20'
    S20AD='S20AD'
    S2MEF='S2MEF'
    S3='S3'
    S3AD='S3AD'
    S3ASR='S3ASR'
    S3D='S3D'
    S3DPOS='S3DPOS'
    S3MEF='S3MEF'
    S4='S4'
    S4AD='S4AD'
    S4EF='S4EF'
    S4EFAD='S4EFAD'
    S4MEF='S4MEF'
    S4POS='S4POS'
    S6='S6'
    S6AD='S6AD'
    S6EL24='S6EL24'
    S6EL24AD='S6EL24AD'
    S8='S8'
    S8POS='S8POS'
    SB='SB'
    SBAD='SBAD'
    SB1='SB1'
    SB1AD='SB1AD'
    SB1MEF='SB1MEF'
    SB2='SB2'
    SB2AD='SB2AD'
    SB2MEF='SB2MEF'
    SBMEF='SBMEF'
    SBSE='SBSE'
    SBSEA='SBSEA'
    SBSEBD='SBSEBD'
    SBSEC='SBSEC'
    SBSEW='SBSEW'
    SC13D='SC13D'
    SC13DAD='SC13DAD'
    SC13E1='SC13E1'
    SC13E1AD='SC13E1AD'
    SC13E3='SC13E3'
    SC13E3AD='SC13E3AD'
    SC13E4='SC13E4'
    SC13E4AD='SC13E4AD'
    SC13E4F='SC13E4F'
    SC13E4FAD='SC13E4FAD'
    SC13G='SC13G'
    SC13GAD='SC13GAD'
    SC14D1='SC14D1'
    SC14D1AD='SC14D1AD'
    SC14D1F='SC14D1F'
    SC14D1FAD='SC14D1FAD'
    SC14D9='SC14D9'
    SC14D9AD='SC14D9AD'
    SC14D9C='SC14D9C'
    SC14D9F='SC14D9F'
    SC14D9FAD='SC14D9FAD'
    SC14F1='SC14F1'
    SC14F1AD='SC14F1AD'
    SC14N='SC14N'
    SC14NAD='SC14NAD'
    SC14NS='SC14NS'
    SC14NSAD='SC14NSAD'
    SCTOC='SCTOC'
    SCTOI='SCTOI'
    SCTOIAD='SCTOIAD'
    SCTOT='SCTOT'
    SCTOTAD='SCTOTAD'
    SD='SD'
    SDAD='SDAD'
    SDR='SDR'
    SDRAD='SDRAD'
    SDRA='SDRA'
    SDRW='SDRW'
    SF1='SF1'
    SF1AD='SF1AD'
    SF3='SF3'
    SF3AD='SF3AD'
    SF3ME='SF3ME'
    SP15D2='SP15D2'
    SP15D2AD='SP15D2AD'
    SUPPL='SUPPL'
    T3='T3'
    T3AD='T3AD'
    T6='T6'
    T6AD='T6AD'
    TA1='TA1'
    TA1AD='TA1AD'
    TA2='TA2'
    TA2AD='TA2AD'
    TAW='TAW'
    U1='U1'
    U1AD='U1AD'
    U12IA='U12IA'
    U12IAAD='U12IAAD'
    U12IB='U12IB'
    U12IBAD='U12IBAD'
    U131='U131'
    U131AD='U131AD'
    U1360='U1360'
    U1360AD='U1360AD'
    U13E1='U13E1'
    U13E1AD='U13E1AD'
    U13El='U13El'
    U13ElAD='U13ElAD'
    U33S='U33S'
    U33SAD='U33SAD'
    U3A2='U3A2'
    U3A2AD='U3A2AD'
    U3A31='U3A31'
    U3A31AD='U3A31AD'
    U57='U57'
    U57AD='U57AD'
    U5A='U5A'
    U5AAD='U5AAD'
    U5B='U5B'
    U5BAD='U5BAD'
    U5S='U5S'
    U5SAD='U5SAD'
    U6B2='U6B2'
    U6B2AD='U6B2AD'
    U7D='U7D'
    U7DAD='U7DAD'
    U9C3='U9C3'
    U9C3AD='U9C3AD'
    UNDER='UNDER'
    UNDERAD='UNDERAD'
    UPLOAD='UPLOAD'
    UR1='UR1'
    UR1AD='UR1AD'
    X17A5='X17A5'
    X17A5AD='X17A5AD'

from enum import Enum


class FilingType(Enum):
    """Available filing types to be used when creating Filing object.
    .. versionadded:: 0.1.5
    """

    FILING_1 = '1'
    FILING_1A = '1-a'
    FILING_1E = '1-e'
    FILING_1K = '1-k'
    FILING_1N = '1-n'
    FILING_1SA = '1-sa'
    FILING_1U = '1-u'
    FILING_1Z = '1-z'
    FILING_10 = '10'
    FILING_10D = '10-d'
    FILING_10K = '10-k'
    FILING_10M = '10-m'
    FILING_10Q = '10-q'
    FILING_11K = '11-k'
    FILING_12B25 = '12b-25'
    FILING_13F = '13f'
    FILING_13H = '13h'
    FILING_144 = '144'
    FILING_15 = '15'
    FILING_15F = '15f'
    FILING_17H = '17-h'
    FILING_18 = '18'
    FILING_18K = '18-k'
    FILING_19B4 = '19b-4'
    FILING_19B4E = '19b-4(e)'
    FILING_19B7 = '19b-7'
    FILING_2E = '2-e'
    FILING_20F = '20-f'
    FILING_24F2 = '24f-2'
    FILING_25 = '25'
    FILING_3 = '3'
    FILING_4 = '4'
    FILING_40F = '40-f'
    FILING_8K = '8-k'
    FILING_ABS15G = 'abs-15g'
    FILING_ABSEE = 'abs-ee'
    FILING_ADV = 'adv'
    FILING_ADVE = 'adv-e'
    FILING_ADVH = 'adv-h'
    FILING_ADVNR = 'adv-nr'
    FILING_ADVW = 'adv-w'
    FILING_ATS = 'ats'
    FILING_ATSN = 'ats-n'
    FILING_ATSR = 'ats-r'
    FILING_BD = 'bd'
    FILING_BDN = 'bd-n'
    FILING_BDW = 'bdw'
    FILING_C = 'c'
    FILING_CA1 = 'ca-1'
    FILING_CB = 'cb'
    FILING_CFPORTAL = 'cfportal'
    FILING_CUSTODY = 'custody'
    FILING_D = 'd'
    FILING_DEF_14A = 'def 14a'
    FILING_F1 = 'f-1'
    FILING_F10 = 'f-10'
    FILING_F3 = 'f-3'
    FILING_F4 = 'f-4'
    FILING_F6 = 'f-6'
    FILING_F7 = 'f-7'
    FILING_F8 = 'f-8'
    FILING_F80 = 'f-80'
    FILING_FN = 'f-n'
    FILING_FX = 'f-x'
    FILING_ID = 'id'
    FILING_MA = 'ma'
    FILING_MAI = 'ma-i'
    FILING_MANR = 'ma-nr'
    FILING_MAW = 'ma-w'
    FILING_MSD = 'msd'
    FILING_MSDW = 'msdw'
    FILING_N14 = 'n-14'
    FILING_N17D1 = 'n-17d-1'
    FILING_N17F1 = 'n-17f-1'
    FILING_N17F2 = 'n-17f-2'
    FILING_N18F1 = 'n-18f-1'
    FILING_N1A = 'n-1a'
    FILING_N2 = 'n-2'
    FILING_N23C3 = 'n-23c-3'
    FILING_N27D1 = 'n27d-1'
    FILING_N3 = 'n-3'
    FILING_N4 = 'n-4'
    FILING_N5 = 'n-5'
    FILING_N54A = 'n-54a'
    FILING_N54C = 'n-54c'
    FILING_N6 = 'n-6'
    FILING_N6EI1 = 'n-6ei-1'
    FILING_N6F = 'n-6f'
    FILING_N8A = 'n-8a'
    FILING_N8B2 = 'n-8b-2'
    FILING_N8B4 = 'n-8b-4'
    FILING_N8F = 'n-8f'
    FILING_NCEN = 'n-cen'
    FILING_NCR = 'n-cr'
    FILING_NCSR = 'n-csr'
    FILING_NCSRS = 'n-csrs'
    FILING_NLIQUID = 'n-liquid'
    FILING_NMFP = 'n-mfp'
    FILING_NPORT = 'nport'
    FILING_NPORTP = 'nport-p'
    FILING_NPORTEX = 'nport-ex'
    FILING_NPX = 'n-px'
    FILING_NQ = 'n-q'
    FILING_NRSRO = 'nrsro'
    FILING_PF = 'pf'
    FILING_PILOT = 'pilot'
    FILING_R31 = 'r31'
    FILING_S1 = 's-1'
    FILING_S11 = 's-11'
    FILING_S20 = 's-20'
    FILING_S3 = 's-3'
    FILING_S4 = 's-4'
    FILING_S6 = 's-6'
    FILING_S8 = 's-8'
    FILING_SBSE = 'sbse'
    FILING_SBSEA = 'sbse-a'
    FILING_SBSEBD = 'sbse-bd'
    FILING_SBSEC = 'sbse-c'
    FILING_SBSEW = 'sbse-w'
    FILING_SCI = 'sci'
    FILING_SD = 'sd'
    FILING_SDR = 'sdr'
    FILING_SE = 'se'
    FILING_SF1 = 'sf-1'
    FILING_SF3 = 'sf-3'
    FILING_SIP = 'sip'
    FILING_T1 = 't-1'
    FILING_T2 = 't-2'
    FILING_T3 = 't-3'
    FILING_T4 = 't-4'
    FILING_T6 = 't-6'
    FILING_TA1 = 'ta-1'
    FILING_TA2 = 'ta-2'
    FILING_TAW = 'ta-w'
    FILING_TCR = 'tcr'
    FILING_TH = 'th'
    FILING_WBAPP = 'wb-app'